@@ -23,6 +23,9 @@ from gi.repository import GtkSource
 from gi.repository import Pango
 
 from gajim.common import app
+from gajim.common import events
+from gajim.common import ged
+from gajim.common.ged import EventHelper
 from gajim.common.i18n import _
 from gajim.common.i18n import get_default_lang
 from gajim.common.storage.archive import models as mod
@@ -53,7 +56,7 @@ FORMAT_CHARS: dict[str, str] = {
 log = logging.getLogger("gajim.gtk.message_input")
 
 
-class MessageInputTextView(GtkSource.View):
+class MessageInputTextView(GtkSource.View, EventHelper):
     """
     A GtkSource.View for chat message input
     """
@@ -75,6 +78,7 @@ class MessageInputTextView(GtkSource.View):
             margin_bottom=3,
             valign=Gtk.Align.CENTER,
         )
+        EventHelper.__init__(self)
 
         self._parent = parent
         self._completion_providers = [
@@ -105,6 +109,9 @@ class MessageInputTextView(GtkSource.View):
         focus_controller.connect("leave", self._on_focus_leave)
         self.add_controller(focus_controller)
 
+        manager = app.app.get_shortcut_manager()
+        manager.install_shortcuts(self, ["input", "input-global"])
+
         self.connect_after("paste-clipboard", self._after_paste_clipboard)
 
         self._speller_menu: Gio.MenuModel | None = None
@@ -112,6 +119,64 @@ class MessageInputTextView(GtkSource.View):
 
         app.plugin_manager.gui_extension_point("message_input", self)
 
+        self.register_events(
+            [
+                ("register-actions", ged.GUI2, self._on_register_actions),
+            ]
+        )
+
+    def _on_register_actions(self, _event: events.RegisterActions) -> None:
+        actions = [
+            "input-clear",
+            "input-focus",
+            "input-bold",
+            "input-italic",
+            "input-strike",
+            "input-emoji",
+            "input-paste-as-quote",
+            "input-paste-as-code-block",
+            "input-mention",
+        ]
+
+        for action in actions:
+            action = app.window.get_action(action)
+            action.connect("activate", self._on_action)
+
+    def _on_action(
+        self, action: Gio.SimpleAction, param: GLib.Variant | None
+    ) -> int | None:
+        if self._contact is None:
+            return
+
+        action_name = action.get_name()
+        log.info("Activate action: %s", action_name)
+
+        match action_name:
+            case "input-clear":
+                self.clear()
+
+            case "input-focus":
+                self.grab_focus_delayed()
+
+            case "input-bold" | "input-italic" | "input-strike":
+                self._apply_formatting(action_name.removeprefix("input-"))
+
+            case "input-emoji":
+                self.emit("insert-emoji")
+
+            case "input-paste-as-quote":
+                self._paste_as_quote()
+
+            case "input-paste-as-code-block":
+                self._paste_as_code_block()
+
+            case "input-mention":
+                assert param is not None
+                self._mention_participant(param.get_string())
+
+            case _:
+                pass
+
     def get_completion_popover(self) -> CompletionPopover:
         return self._completion_popover
 
@@ -214,18 +279,18 @@ class MessageInputTextView(GtkSource.View):
         _n_press: int,
         x: float,
         y: float,
-    ) -> int:
+    ) -> None:
         # Place the cursor at click position to trigger an update
         # for spelling suggestions, see:
         # https://gitlab.gnome.org/GNOME/libspelling/-/issues/5
         buf = self.get_buffer()
         if buf.get_has_selection():
             # Don't place cursor if text is selected, otherwise the selection is changed
-            return Gdk.EVENT_PROPAGATE
+            return
 
         _, iter_, _ = self.get_iter_at_position(int(x), int(y))
         buf.place_cursor(iter_)
-        return Gdk.EVENT_PROPAGATE
+        return
 
     def _on_focus_enter(self, _focus_controller: Gtk.EventControllerFocus) -> None:
         scrolled = self.get_parent()
@@ -317,7 +382,7 @@ class MessageInputTextView(GtkSource.View):
             start, end = buf.get_bounds()
         return (start, end)
 
-    def apply_formatting(self, formatting: str) -> None:
+    def _apply_formatting(self, formatting: str) -> None:
         format_char = FORMAT_CHARS[formatting]
 
         buf = self.get_buffer()
@@ -357,8 +422,11 @@ class MessageInputTextView(GtkSource.View):
 
     def clear(self, *args: Any) -> None:
         buf = self.get_buffer()
-        start, end = buf.get_bounds()
-        buf.delete(start, end)
+        buf.delete(*buf.get_bounds())
+
+        # Action only available in Gtk 4.19
+        # Use this as soon we set it as min Gtk
+        # self.activate_action("text.clear")
 
     def undo(self, *args: Any) -> None:
         buf = self.get_buffer()
@@ -370,7 +438,7 @@ class MessageInputTextView(GtkSource.View):
         if buf.get_can_redo():
             buf.redo()
 
-    def mention_participant(self, name: str) -> None:
+    def _mention_participant(self, name: str) -> None:
         gc_refer_to_nick_char = app.settings.get("gc_refer_to_nick_char")
         text = f"{name}{gc_refer_to_nick_char} "
         self.insert_text(text)
@@ -385,16 +453,16 @@ class MessageInputTextView(GtkSource.View):
         self.insert_text(f"```\n{text}\n```")
         self.grab_focus()
 
-    def paste_as_quote(self) -> None:
+    def _paste_as_quote(self) -> None:
         clipboard = self.get_clipboard()
         clipboard.read_text_async(
-            None, self._on_clipboard_read_text_finished, "paste-as-quote"
+            None, self._on_clipboard_read_text_finished, "input-paste-as-quote"
         )
 
-    def paste_as_code_block(self) -> None:
+    def _paste_as_code_block(self) -> None:
         clipboard = self.get_clipboard()
         clipboard.read_text_async(
-            None, self._on_clipboard_read_text_finished, "paste-as-code-block"
+            None, self._on_clipboard_read_text_finished, "input-paste-as-code-block"
         )
 
     def _on_clipboard_read_text_finished(
@@ -410,9 +478,9 @@ class MessageInputTextView(GtkSource.View):
             log.info("No text pasted")
             return
 
-        if action_name == "paste-as-quote":
+        if action_name == "input-paste-as-quote":
             self.insert_as_quote(text)
-        elif action_name == "paste-as-code-block":
+        elif action_name == "input-paste-as-code-block":
             self.insert_as_code_block(text)
 
 
@@ -445,7 +513,10 @@ class TextBufferManager(GObject.Object):
     def switch_contact(self, contact: ChatContactT) -> None:
         buf = self._text_buffers.get(contact)
         if buf is None:
-            buf = GtkSource.Buffer()
+            buf = GtkSource.Buffer(
+                implicit_trailing_newline=False,
+                highlight_matching_brackets=False,
+            )
             buf.create_tag("strong", weight=Pango.Weight.BOLD)
             buf.create_tag("emphasis", style=Pango.Style.ITALIC)
             buf.create_tag("strike", strikethrough=True)
