@@ -205,8 +205,7 @@ class AccountPrivacyGroup(GajimPreferencesGroup):
                 SettingType.ACCOUNT_CONFIG,
                 "encryption_default",
                 desc=_(
-                    "Encryption method to use "
-                    "unless overridden on a per-contact basis"
+                    "Encryption method to use unless overridden on a per-contact basis"
                 ),
                 props={"data": encryption_entries},
             ),
@@ -225,8 +224,7 @@ class AccountPrivacyGroup(GajimPreferencesGroup):
                 "send_os_info",
                 callback=self._send_os_info,
                 desc=_(
-                    "Disclose information about the "
-                    "operating system you currently use"
+                    "Disclose information about the operating system you currently use"
                 ),
             ),
             Setting(
@@ -286,7 +284,7 @@ class AccountPrivacyGroup(GajimPreferencesGroup):
             ),
             Setting(
                 SettingKind.SWITCH,
-                _("Send Read Markers"),
+                _("Read Receipts"),
                 SettingType.VALUE,
                 app.settings.get_account_setting(account, "send_marker_default"),
                 callback=self._send_read_marker,
@@ -599,7 +597,7 @@ class AccountBlockedContactsGroup(GajimPreferencesGroup):
 class AccountArchivingGroup(GajimPreferencesGroup):
     def __init__(self, account: str) -> None:
         GajimPreferencesGroup.__init__(
-            self, key="account-archiving", account=account, title=_("Archiving")
+            self, key="account-archiving", account=account, title=_("Archive")
         )
 
         settings = [
@@ -628,6 +626,11 @@ class AccountArchivingPreferencesGroup(GajimPreferencesGroup):
             self,
             key="account-archiving-preferences",
             account=account,
+            description=_(
+                "Configure which messages will be archived on the server. "
+                "Not archiving messages on the server may prevent them from "
+                "synchronizing between your devices."
+            ),
             title=_("Preferences"),
         )
 
@@ -1045,12 +1048,10 @@ class AccountActiveSwitch(Adw.ActionRow, SignalManager, EventHelper):
     def _on_state_changed(
         self, client: types.Client, _signal_name: str, client_state: ClientState
     ) -> None:
-
         if client_state.is_disconnected:
             app.app.disable_account(client.account)
 
     def _on_state_set(self, switch: Gtk.Switch, state: bool, account: str) -> int:
-
         def _on_response(response_id: str) -> None:
             if response_id == "disable":
                 client = app.get_client(account)
