@@ -24,7 +24,7 @@ from gajim.gtk.audio_player import AudioPlayer
 from gajim.gtk.filechoosers import FileChooserButton
 from gajim.gtk.filechoosers import Filter
 from gajim.gtk.preview.audio import AudioPreviewWidget
-from gajim.gtk.widgets import GajimAppWindow
+from gajim.gtk.window import GajimAppWindow
 
 from . import util
 
@@ -39,6 +39,8 @@ class TestAudioWidget(GajimAppWindow):
             title=__class__.__name__,
             default_width=600,
             default_height=600,
+            add_window_padding=True,
+            header_bar=True,
         )
 
         self._box = Gtk.Box(
@@ -50,6 +52,8 @@ class TestAudioWidget(GajimAppWindow):
         )
         self.set_child(self._box)
 
+        self._audio_player = AudioPlayer()
+
         file_chooser_button = FileChooserButton(
             label="Select Audio File",
             filters=[
@@ -63,6 +67,7 @@ class TestAudioWidget(GajimAppWindow):
         self._box.append(file_chooser_button)
 
         self._audio_widget = AudioPreviewWidget(
+            self._audio_player,
             DEFAULT_AUDIO_FILE_PATH.name,
             DEFAULT_AUDIO_FILE_PATH.stat().st_size,
             DEFAULT_AUDIO_FILE_PATH,
@@ -74,10 +79,13 @@ class TestAudioWidget(GajimAppWindow):
         del self._audio_widget
 
         self._audio_widget = AudioPreviewWidget(
-            paths[0].as_posix(), paths[0].stat().st_size, paths[0]
+            self._audio_player, paths[0].as_posix(), paths[0].stat().st_size, paths[0]
         )
         self._box.append(self._audio_widget)
 
+    def _cleanup(self) -> None:
+        pass
+
 
 util.init_settings()
 
@@ -86,7 +94,6 @@ Gst.init()
 app.is_installed = MagicMock(return_value=True)
 
 app.init_process_pool()
-app.audio_player = AudioPlayer()
 
 window = TestAudioWidget()
 window.show()
