@@ -44,8 +44,10 @@ def get_build_args() -> list[str]:
         os.path.dirname(sys.executable), python_name + "-config"
     )
 
-    cflags = subprocess.check_output(["sh", python_config, "--cflags"]).strip()
-    libs = subprocess.check_output(["sh", python_config, "--libs"]).strip()
+    cflags = subprocess.check_output(
+        [sys.executable, python_config, "--cflags"]
+    ).strip()
+    libs = subprocess.check_output([sys.executable, python_config, "--libs"]).strip()
 
     cflags = os.fsdecode(cflags)
     libs = os.fsdecode(libs)
